import logging
import yaml


class Config:
    def __init__(self, configfile):
        with open(configfile, 'r') as config:
            document = yaml.safe_load(config)
        self.username = document["Connection"]["Username"]
        self.password = document["Connection"]["Password"]
        self.logging = document["LogLevel"]
        self.PVWA = document["PVWA"]
        self.CA = document["CAFile"]


DEFAULT_PERMISSIONS = {
        "UseAccounts": False,
        "RetrieveAccounts": False,
        "ListAccounts": False,
        "AddAccounts": False,
        "UpdateAccountContent": False,
        "UpdateAccountProperties": False,
        "InitiateCPMAccountManagementOperations": False,
        "SpecifyNextAccountContent": False,
        "RenameAccounts": False,
        "DeleteAccounts": False,
        "UnlockAccounts": False,
        "ManageSafe": False,
        "ManageSafeMembers": False,
        "BackupSafe": False,
        "ViewAuditLog": False,
        "ViewSafeMembers": False,
        "RequestsAuthorizationLevel": 0,
        "AccessWithoutConfirmation": False,
        "CreateFolders": False,
        "DeleteFolders": False,
        "MoveAccountsAndFolders": False
    }

PROV_PERMISSIONS = {
        "UseAccounts": False,
        "RetrieveAccounts": False,
        "ListAccounts": True,
        "AddAccounts": True,
        "UpdateAccountContent": True,
        "UpdateAccountProperties": True,
        "InitiateCPMAccountManagementOperations": False,
        "SpecifyNextAccountContent": False,
        "RenameAccounts": True,
        "DeleteAccounts": True,
        "UnlockAccounts": False,
        "ManageSafe": True,
        "ManageSafeMembers": True,
        "BackupSafe": False,
        "ViewAuditLog": False,
        "ViewSafeMembers": True,
        "RequestsAuthorizationLevel": 0,
        "AccessWithoutConfirmation": True,
        "CreateFolders": True,
        "DeleteFolders": True,
        "MoveAccountsAndFolders": True
    }

ADMIN_PERMISSIONS = {
    "UseAccounts": True,
    "RetrieveAccounts": True,
    "ListAccounts": True,
    "AddAccounts": True,
    "UpdateAccountContent": True,
    "UpdateAccountProperties": True,
    "InitiateCPMAccountManagementOperations": True,
    "SpecifyNextAccountContent": True,
    "RenameAccounts": True,
    "DeleteAccounts": True,
    "UnlockAccounts": True,
    "ManageSafe": True,
    "ManageSafeMembers": True,
    "BackupSafe": True,
    "ViewAuditLog": True,
    "ViewSafeMembers": True,
    "RequestsAuthorizationLevel": 0,
    "AccessWithoutConfirmation": True,
    "CreateFolders": True,
    "DeleteFolders": True,
    "MoveAccountsAndFolders": True
}


def validate_ip(s):
    a = s.split('.')
    if len(a) != 4:
        return False
    for x in a:
        if not x.isdigit():
            return False
        i = int(x)
        if i < 0 or i > 255:
            return False
    return True


def permissions(profile: str) -> list:
    if "admin" in profile.lower():
        return cast_permissions(ADMIN_PERMISSIONS)
    if "prov" in profile.lower():
        return cast_permissions(PROV_PERMISSIONS)
    else:
        return cast_permissions(DEFAULT_PERMISSIONS)


def cast_permissions(perm: dict) -> list:
    permission = []
    for right, value in perm.items():
        permission.append({"Key": right, "Value": value})
    return permission



