import requests
import json
from bastion.exceptions import CyberarkException, GetTokenException
from bastion.config import Config

class EPV:

    def __init__(self, configfile: str):
        self.config = Config(configfile)
        self.token = None
        self.api_host = self.config.PVWA
        self.session = requests.Session()
        self.cpm = "" #todo add in config
        self.retention = 30 #todo add in config
        self.user_list = None
        self.username = self.config.username
        self.__password = self.config.password

        # utilities
        self.account = self.Account(self)
        self.platform = self.Platform(self)
        print("instanciation de la classe EPV realisee")

    class Account:
        def __init__(self, s):
            pass

    class Platform:
        def __init__(self, s):
            pass

    #def __del__(self):
        #self.logoff()

    # def get_api_host(self):
    #     return self.api_host

    # def get_token(self):
    #     return self.__token

    # def get_session(self):
    #     return self.session

    def __login_cyberark(self, username: str, password: str) -> str:
        url = 'https://' + self.api_host + '/PasswordVault/API/Auth/CyberArk/Logon'
        request_data = {"username": username, "password": password}
        head = {"Content-type": "application/json"}
        try:
            req = self.session.post(url, headers=head, data=json.dumps(request_data), timeout=30, verify=False)
            req.raise_for_status()
        except requests.exceptions.HTTPError as http_err:
            raise CyberarkException("(HTTP Error)Unable to connect to Cyberark API", http_err)
        except Exception as err:
            raise CyberarkException("(Other Error) Unable to process request to Cyberark API", err)
        return json.loads(req.content)

    def logoff(self):
        url = 'https://' + self.api_host + '/PasswordVault/API/Auth/Logoff'
        head = {"Content-type": "application/json", "Authorization": self.__token}
        try:
            req = self.session.post(url, headers=head, timeout=30, verify=False)
            req.raise_for_status()
        except requests.exceptions.HTTPError as http_err:
            raise CyberarkException("Raised HTTP ERROR in login_cyberark function ")
        except Exception as err:
            raise CyberarkException("Unknown error while logging off")
        self.__token = None

    def __check_token(self) -> bool:
        if self.__token is None:
            return False
        url = 'https://' + self.api_host + '/PasswordVault/WebServices/PIMServices.svc/User'
        head = {"Content-type": "application/json", "Authorization": self.__token}
        req = self.session.get(url, headers=head, timeout=30, verify=False)
        if req.status_code != 200:
            self.__token = None
            return False
        return True


    def login(self):
        if self.__check_token():
            return
        try:
            #TODO add in aim
            #account = self.get_password_object(self.__object)
            #user = account['UserName']
            #password = account['Content']
            user = self.username
            password = self.__password
        except PermissionError:
            raise GetTokenException("I was unable to grab password")
        except OSError:
            raise GetTokenException("i was unable to read the tmp file ")

        try:
            self.__token = self.__login_cyberark(user, password)
        except CyberarkException as err:
            raise GetTokenException("Check client credentials or if user is not connected on PrivateArk Client")


    def get_url(self, url)-> (str, str):
        addr = 'https://' + self.api_host + '/PasswordVault/' + url
        head = {"Content-type": "application/json", "Authorization": self.token}
        return addr, head

