import json

from ..cyberark import EPV
from ..exceptions import CyberarkException, CyberarkAPIException
import shutil

class Platform:
    def __init__(self, epv: EPV):
        self.epv = epv
        self.session = self.epv.session


    def get_platforms(self):
        # url = 'https://' + self.epv.api_host + '/PasswordVault/API/Platforms/Targets'
        # head = {"Content-type": "application/json", "Authorization": self.epv.token}

        url, head = self.epv.get_url("API/Platforms/Targets")

        try:
            req = self.session.get(url, headers=head, timeout=30, verify=False)
        except Exception as err:
            # uncatched exception
            raise CyberarkException(err)

        if req.status_code != 200:
            content = json.loads(req.content)
            raise CyberarkAPIException(content)

        platforms = json.loads(req.content)
        return platforms["Platforms"]


    def del_plaform(self, pfid):
        url, head = self.epv.get_url("API/Platforms/Targets/"+str(pfid))

        try:
            req = self.session.delete(url, headers=head, timeout=30, verify=False)
            #req = epv.session.get(url, headers=head, params=data, timeout=30, verify=False)
        except Exception as err:
            # uncatched exception
            raise CyberarkException(err)

        if req.status_code != 204:
            content = json.loads(req.content)
            #raise CyberarkAPIException(content["ErrorMessage"])
            raise CyberarkAPIException(content)

        return True

    def export_platform(self, pfid: str):
        url, head = self.epv.get_url("API/Platforms/"+str(pfid)+"/Export")

        try:
            req = self.session.post(url, headers=head, data=None, timeout=30, verify=False, stream=True)
        except Exception as err:
            # uncatched exception
            raise CyberarkException(err)

        if req.status_code != 200:
            content = json.loads(req.content)
            raise CyberarkAPIException(content["Message"])

        with open("../out/"+pfid+".zip", "wb") as pf_file:
            shutil.copyfileobj(req.raw, pf_file)

        return "out/"+pfid+".zip"
